<?php

namespace Ems\portalGestor\Repository;

use Doctrine\ORM\EntityManager;

class DemonstrativoRepository
{
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
    }

    public function getDiaDemonstrativo(array $params): array
    {
        $sql = <<<SQL
            SELECT apu_dia.*
            FROM TABLE ( pontodemonst_dados(TO_DATE(:p_dtmescompetencia, 'DD/MM/YYYY'),
                :p_nrorg, TO_DATE(:p_dtapuracaoponto, 'DD/MM/YYYY'),
                :p_nrperiodoapuracao, :p_nrvinculom) ) apu_dia
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $demonstrativo = $std->fetch();

        return $demonstrativo;
    }


    public function getPeriodosDemonstrativo(array $params): array
    {
        $sql = <<<SQL
            SELECT *
            FROM   (SELECT PPA.NRPERIODOAPURACAO,
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')
                        || ' - '
                        || TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')
                        || ' ('
                        || PPA.IDPERIODOAPURACAO
                        || ')' AS DSPERIODOAPURACAO,
                        PPA.NRORG,
                        TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY') AS DTFINALAPURACAO,
                        TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY') AS DTINICIALAPURACAO,
                        PPA.NRTIPOPERIODOAPURACAO,
                        PPA.IDPERIODOAPURACAO
                    FROM   PTO_PERIODOAPURACAO      PPA,
                           PTO_TIPOPERIODOAPURACAO  TPA
                    WHERE  PPA.NRORG = :P_NRORG
                        AND TPA.NRTIPOVINCULOM = 1 /* OS PERÍODOS RETORNADOS SERÃO SEMPRE OS PERÍODOS DE CLT */
                        AND PPA.DTINICIALAPURACAO <= LAST_DAY(SYSDATE)
                        AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                        AND PPA.NRORG = TPA.NRORG
                    ORDER  BY PPA.DTINICIALAPURACAO DESC)
            WHERE  ROWNUM <= 12
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodos = $std->fetchAll();

        return $periodos;
    }

    public function getPeriodoDemonstrativo(array $params): array
    {
        $sql = <<<SQL
                SELECT PPA.nrperiodoapuracao,
                    To_char(PPA.dtinicialapuracao, 'DD/MM/YYYY')
                    || ' - '
                    || To_char(PPA.dtfinalapuracao, 'DD/MM/YYYY')
                    || ' ('
                    || PPA.idperiodoapuracao
                    || ')' AS DSPERIODOAPURACAO,
                    PPA.nrorg,
                    TO_CHAR(PPA.dtfinalapuracao, 'DD/MM/YYYY') AS DTFINALAPURACAO,
                    TO_CHAR(PPA.dtinicialapuracao, 'DD/MM/YYYY') AS DTINICIALAPURACAO,
                    PPA.nrtipoperiodoapuracao,
                    PPA.idperiodoapuracao
                FROM   pto_periodoapuracao PPA,
                    pto_tipoperiodoapuracao TPA
                WHERE  PPA.nrorg = :p_nrorg
                    AND PPA.nrperiodoapuracao = :p_nrperiodoapuracao
                    AND PPA.nrtipoperiodoapuracao = TPA.nrtipoperiodoapuracao
                    AND PPA.nrorg = TPA.nrorg
                ORDER  BY PPA.dtinicialapuracao DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodos = $std->fetchAll();

        return $periodos;
    }

    public function getAllDateIds(array $params): array {
        $sql = <<<SQL
            SELECT IDDIAOCORRENCIA IDDIA
            FROM GPE_MARCACAOPONTO
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULO
              AND IDMARCDESPREZADA = 'N'
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA = 'DC')
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DATE)
                   OR (IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY')))
            ORDER BY DTHRAJUSTADA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $dateIds = $std->fetchAll();

        return $dateIds;
    }

    public function getPeriodoDemonstrativoPorTipoVinculo(array $params): array
    {
        $sql = <<<SQL
                SELECT PPA.nrperiodoapuracao,
                    To_char(PPA.dtinicialapuracao, 'DD/MM/YYYY')
                    || ' - '
                    || To_char(PPA.dtfinalapuracao, 'DD/MM/YYYY')
                    || ' ('
                    || PPA.idperiodoapuracao
                    || ')' AS DSPERIODOAPURACAO,
                    PPA.nrorg,
                    TO_CHAR(PPA.dtfinalapuracao, 'DD/MM/YYYY') AS DTFINALAPURACAO,
                    TO_CHAR(PPA.dtinicialapuracao, 'DD/MM/YYYY') AS DTINICIALAPURACAO,
                    PPA.nrtipoperiodoapuracao,
                    PPA.idperiodoapuracao
                FROM   pto_periodoapuracao PPA,
                    pto_tipoperiodoapuracao TPA
                WHERE   PPA.nrorg = :p_nrorg
                    AND PPA.dtfinalapuracao = to_date(:p_dtfinalapuracao, 'DD/MM/YYYY')
                    AND PPA.dtinicialapuracao = to_date(:p_dtinicialapuracao, 'DD/MM/YYYY')
                    AND TPA.nrtipovinculom = :p_nrtipovinculom

                    AND PPA.nrtipoperiodoapuracao = TPA.nrtipoperiodoapuracao
                    AND PPA.nrorg = TPA.nrorg
                ORDER  BY PPA.dtinicialapuracao DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodos = $std->fetchAll();

        return $periodos;
    }

    public function getMotivosOcorrencia(array $params): array
    {
        $sql = <<<SQL
            SELECT
              NRMOTIVOOCORFREQ,
              DSMOTIVOOCORFREQ,
              IDTIPOINTERVALO
            FROM PTO_MOTIVOOCORFREQ
            WHERE (NRORG = 0 OR NRORG = :P_NRORG)
              AND IDATIVO = 'S'
            ORDER BY DSMOTIVOOCORFREQ
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getOcorrenciasDoDia($params)
    {
        $sql = <<<SQL
            SELECT *
            FROM (
                SELECT OCOR.NRMOTIVOOCORFREQ,
                    OCOR.IDPROGOCORCONFIRMADA,
                    MOT.DSMOTIVOOCORFREQ
                FROM PTO_PROGOCORRENCIA OCOR
                    LEFT JOIN PTO_MOTIVOOCORFREQ MOT
                                ON MOT.NRMOTIVOOCORFREQ = OCOR.NRMOTIVOOCORFREQ
                                AND MOT.NRORG = OCOR.NRORG
                WHERE OCOR.NRVINCULOM = :P_NRVINCULOM
                AND OCOR.NRORG = :P_NRORG
                AND TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') BETWEEN
                        DTINICIOPROGOCOR AND DTFIMPROGOCOR
                AND OCOR.NRMOTIVOOCORFREQ IS NOT NULL )
            WHERE ROWNUM = 1
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $ocorrencias = $std->fetchAll();

        return $ocorrencias;
    }

    public function verificaOcorrenciasCalendario(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {
        $sql = <<<SQL
            SELECT
              COUNT(*) AS COUNT_OCORRENCIAS_FERIADO
              /*OCORCALENDARIO.DTCALENDARIO,
              OCORCALENDARIO.DSOBSCALEND,
              TPCALENDARIO.DSTPCALENDARIO,
              TPCALENDARIO.DSABREVTPCALEND,
              TPCALENDARIO.IDTIPODIA*/
             FROM OCORCALENDARIO
             JOIN TPCALENDARIO
               ON TPCALENDARIO.NRTPCALENDARIO = OCORCALENDARIO.NRTPCALENDARIO
            WHERE OCORCALENDARIO.NRORG = :P_NRORG
              AND OCORCALENDARIO.DTCALENDARIO = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND OCORCALENDARIO.NRCALENDARIO IN (SELECT NRCALENDARIO
                                    FROM ESTRUTURAH
                                    WHERE NRESTRUTURAM = (
                                        SELECT MOV.NRESTRUTURAM
                                        FROM GPE_MOVIMENTACAO MOV
                                        JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                                        WHERE MOV.NRORG = :P_NRORG
                                            AND MOV.NRVINCULOM = :P_NRVINCULOM
                                            AND MOV.DTINIMOVIMENT <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
                                            AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT >  LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')))
                                            AND MOV.NRTIPOESTRUTURA IN (PO.NRTPESTRUTLEGAL, PO.NRTPESTRUTCALEND)
                                        ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                                        FETCH FIRST ROW ONLY
                                   )
                                    AND DTMESCOMPETENC IN (
                                        SELECT MAX(DTMESCOMPETENC)
                                           FROM ESTRUTURAH H1
                                          WHERE H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                            AND H1.NRORG = ESTRUTURAH.NRORG
                                            AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                                        )
                                    AND NRORG = :P_NRORG)
              AND OCORCALENDARIO.NRORG = :P_NRORG
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
                "P_DTMESCOMPETENCIA" => $dtmescompetencia
            )
        );

        $ocorrenciasCalendario = $query->fetchAll();
        if (isset($ocorrenciasCalendario[0])) {
            return $ocorrenciasCalendario[0];
        } else {
            return null;
        }
    }
}
